from rest import decorators as rd
from rest import views as rv
from rest import helpers as rh
from . import models as am
from .parsers import ossec


@rd.urlPOST(r'^ossec/alert$')
def ossec_alert_creat_from_request(request):
    payload = request.DATA.get("payload")
    if payload:
        try:
            # TODO make this a task (background it)
            od = ossec.parseAlert(request, payload)
            # lets now create a local event
            if od is not None:
                level = 10
                if od.level > 10:
                    level = 1
                elif od.level > 7:
                    level = 2
                elif od.level == 6:
                    level = 3
                elif od.level == 5:
                    level = 4
                elif od.level == 4:
                    level = 6
                elif od.level <= 3:
                    level = 8
                metadata = od.toDict(graph="default")
                if od.geoip:
                    metadata["country"] = od.geoip.country
                am.Event.createFromDict(None, {
                    "hostname": od.hostname,
                    "description": od.title,
                    "details": od.text,
                    "level": level,
                    "category": "ossec",
                    "component": "incident.ServerOssecAlert",
                    "component_id": od.id,
                    "metadata": metadata
                })
        except Exception:
            rh.log_exception("during ossec alert", payload)
    return rv.restStatus(request, False, error="no alert data")


@rd.urlGET(r'^ossec$')
@rd.urlGET(r'^ossec/(?P<pk>\d+)$')
@rd.login_required
def on_ossec(request, pk=None):
    return am.ServerOssecAlert.on_rest_request(request, pk)


@rd.urlPOST(r'^event$')
def rest_on_create_event(request, pk=None):
    # TODO check for key?
    resp = am.Event.on_rest_request(request)
    return rv.restStatus(request, True)


@rd.urlGET(r'^event$')
@rd.urlGET(r'^event/(?P<pk>\d+)$')
@rd.login_required
def rest_on_event(request, pk=None):
    return am.Event.on_rest_request(request, pk)


@rd.url(r'^incident$')
@rd.url(r'^incident/(?P<pk>\d+)$')
@rd.login_required
def rest_on_incident(request, pk=None):
    return am.Incident.on_rest_request(request, pk)


@rd.url(r'^incident/history$')
@rd.url(r'^incident/history/(?P<pk>\d+)$')
@rd.login_required
def rest_on_incident_history(request, pk=None):
    return am.IncidentHistory.on_rest_request(request, pk)


@rd.url(r'^rule$')
@rd.url(r'^rule/(?P<pk>\d+)$')
@rd.login_required
def rest_on_rule(request, pk=None):
    return am.Rule.on_rest_request(request, pk)


@rd.url(r'^rule/check$')
@rd.url(r'^rule/check/(?P<pk>\d+)$')
@rd.login_required
def rest_on_rule_check(request, pk=None):
    return am.RuleCheck.on_rest_request(request, pk)




from rest.decorators import periodic
from datetime import datetime, timedelta
from .models import Member
from sessionlog.models import SessionLog


@periodic(minute=15)
def run_cleanup_tokens(force=False, verbose=False, now=None):
    # we want to nuke invite tokens every 15 minutes
    # we do not want to do this if using invite
    stale = datetime.now() - timedelta(hours=24)
    qset = Member.objects.filter(auth_token__isnull=False).filter(modified__lte=stale)
    qset.update(auth_token=None)

    # lets prune old non active sessions
    SessionLog.Clean(limit=10000)




from objict import objict

META_KEYS = ["SERVER_PROTOCOL", "REQUEST_METHOD", "QUERY_STRING", "HTTP_USER_AGENT"]


def event(category, description, level=10, request=None, **kwargs):
    from taskqueue.models import Task
    data = objict(category=category, description=description, level=level)
    data.metadata = objict.fromdict(kwargs)
    Task.Publish("incident", "new_event", channel="tq_app_handler", data=data)


def event_now(category, description, level=10, request=None, **kwargs):
    from .models.event import Event
    data = objict(category=category, description=description, level=level)
    data.metadata = objict.fromdict(kwargs)
    if request is not None:
        data.metadata.ip = request.ip
        if "path" not in data.metadata:
            data.metadata.path = request.path
        for key in META_KEYS:
            value = request.META.get(key, None)
            if value is not None and isinstance(value, str) and "." not in key:
                data.metadata[key.lower()] = str(value)
    if "hostname" in data.metadata:
        data.hostname = data.metadata.hostname
    if "details" in data.metadata:
        data.details = data.metadata.details
    if "component" in data.metadata:
        data.component = data.metadata.component
    if "component_id" in data.metadata:
        data.component_id = data.metadata.component_id
    if "ip" in data.metadata:
        data.reporter_ip = data.metadata.ip
    Event.createFromDict(None, data)



from rest.decorators import periodic
from taskqueue.models import Task


@periodic(hour=12, minute=30)
def run_log_cleanup(force=False, verbose=False, now=None):
    # schedule pruning
    Task.Publish("auditlog", "on_cleanup", channel="tq_app_handler_cleanup")






from django.db import models
from rest import models as rm
from objict import objict
from datetime import datetime

CM_BACKENDS = objict()


class MemberDevice(models.Model, rm.RestModel, rm.MetaDataModel):
    """
    MemberDevice Model tracks personal devices associated with a user.
    This can include mobile and desktop devices.
    """
    created = models.DateTimeField(auto_now_add=True)
    modified = models.DateTimeField(auto_now=True)

    member = models.ForeignKey("account.Member", related_name="devices", on_delete=models.CASCADE)

    name = models.CharField(max_length=128, blank=True, null=True, default=None)
    uuid = models.CharField(db_index=True, max_length=128, blank=True, null=True, default=None)

    cm_provider = models.CharField(db_index=True, max_length=64, default="fcm")
    cm_token = models.CharField(max_length=250, default=None, null=True)

    kind = models.CharField(db_index=True, max_length=64, default="unknown")
    state = models.IntegerField(db_index=True, default=1)

    class RestMeta:
        GRAPHS = {
            "default": {
                "extra": ["metadata"],
            }
        }

    def sendData(self, message, **kwargs):
        messenger = getCloudMessanger(self.cm_provider)
        if messenger:
            return messenger.sendToDevice(self, message)
        return objict(status_code=404, reason=self.cm_provider)

    def sendNotification(self, title, body):
        messenger = getCloudMessanger(self.cm_provider)
        if messenger:
            return messenger.sendNotification(self.cm_token, title, body)
        return objict(status_code=404, reason=self.cm_provider)

    def notify(self, title, body):
        return self.sendNotification(title, body)

    def touch(self):
        self.modified = datetime.now()
        self.save()

    @classmethod
    def sendMessageTo(cls, message, devices, **kwargs):
        pass

    @classmethod
    def register(cls, request, member, device_id):
        cm_token = request.DATA.get("cm_token")
        default_provider = "ws"
        if cm_token is not None:
            default_provider = "fcm"
        cm_provider = request.DATA.get("cm_provider", default=default_provider)
        md = MemberDevice.objects.filter(uuid=device_id, member=member).last()
        if md is not None:
            return md
        md = MemberDevice(uuid=device_id, member=member, cm_token=cm_token, cm_provider=cm_provider)
        md.name = F"{member.first_name} {request.auth_session.device}"
        md.kind = request.auth_session.os.lower()
        if md.kind.startswith("mac"):
            md.kind = "mac"
        md.save()
        md.setProperty("user_agent", request.META.get('HTTP_USER_AGENT', ''))
        return md


class MemberDeviceMetaData(rm.MetaDataBase):
    parent = models.ForeignKey(MemberDevice, related_name="properties", on_delete=models.CASCADE)


def getCloudMessanger(name):
    if name not in CM_BACKENDS:
        if name == "fcm":
            from account import fcm
            CM_BACKENDS["fcm"] = fcm
    return CM_BACKENDS[name]

